/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             exceptions management
*  SCOPE :              
*  FILE :               yooexcept.hxx
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma interface
#endif
#ifndef __yooexcept_HXX
#define __yooexcept_HXX

/* IMPORTS *******************************************************************/

#ifdef __cplusplus

extern "C" {
#ifndef __setjmp_H
#define __setjmp_H
#include <setjmp.h>
#endif
} ;

#ifdef __GLOBAL_TOOLS_DEFINITION__
#define EXTERN_TOOLS(x) x
#define EXTERN_INIT_TOOLS(x ,y) x = y
#else
#define EXTERN_TOOLS(x) extern x
#define EXTERN_INIT_TOOLS(x ,y) extern x
#endif


/*****************************************************************************/

#define TRY  setjmp(env_retry) ; if(!(__YOO_ERR = setjmp(stack())))
#define THROW(x) {\
  __YOO_ERR = x; \
  if (emptyStack()) { \
     yoo_error(x, "Lancement d'exception non recu : ") ;\
     yoo_exit(1) ; \
  } \
  if (setjmp(env_ignore)==0) longjmp(unstack(),__YOO_ERR); }
#define EXCEPTION if (__YOO_ERR == 0) { unstack() ;} else
#define RETRY longjmp(env_retry,0) 
#define IGNORE longjmp(env_ignore,20) 
#define UP { __YOO_ERR = 1; if (emptyStack() == 0) longjmp(unstack(),1) ; else yoo_exit(1) ; }

struct s_Env_stack {
  jmp_buf env[20] ;
  int     head  ;
} ;

typedef struct s_Env_stack Env_stack ;

EXTERN_TOOLS(jmp_buf env_retry) ;
EXTERN_TOOLS(jmp_buf env_ignore) ;
EXTERN_TOOLS(Env_stack env_stack) ;
EXTERN_INIT_TOOLS(int __YOO_ERR, 0) ;

#if defined sun
inline int* stack() {
   return (int*)env_stack.env[env_stack.head++] ;
}
inline int* unstack() {
   return (int*)env_stack.env[--env_stack.head] ;
}
#elif defined __alpha
inline long* stack() {
   return (long*)env_stack.env[env_stack.head++] ;
}
inline long* unstack() {
   return (long*)env_stack.env[--env_stack.head] ;
}
#elif defined hpux
inline double* stack() {
   return (double*)env_stack.env[env_stack.head++] ;
}
inline double* unstack() {
   return (double*)env_stack.env[--env_stack.head] ;
}
#endif

inline int emptyStack() {

  return (env_stack.head == 0) ;
}

extern void call_throw(int, char*) ;
extern void call_throw(int) ;
extern void yoo_sethook(void (*hook)()) ;


#else /* __cplusplus */

extern void call_throw_c(int) ;

#endif /* __cplusplus */

/* Error */
#define YOO_ERR_NONE 0
#define YOO_ERR_DBG  1 
#define YOO_ERR_SEGV 2
#define YOO_ERR_LOCK 3
#define YOO_ERR_DEADLOCK 4
#define YOO_ERR_UPG 5
#define YOO_ERR_MEM 6
#define YOO_ERR_VOL 7
#define YOO_ERR_COM 8
#define YOO_ERR_ADM 9
#define YOO_ERR_COL 10
#define YOO_ERR_STR 11
#define YOO_ERR_READ 12
#define YOO_ERR_BASE 13
#define YOO_ERR_SCHEMA 14
#define YOO_ERR_SIGPIPE 15
#define YOO_ERR_SIGURG 16
#define YOO_ERR_BUS 17

#endif

